"""Contains constants."""
from __future__ import annotations

from typing import Final

STATE_ON: Final = "on"
STATE_OFF: Final = "off"

ADDR_BROADCAST: Final = 0
ADDR_ECONET: Final = 86

ATTR_NAME: Final = "name"
ATTR_VALUE: Final = "value"
ATTR_EXTRA: Final = "extra"
ATTR_BOILER_SENSORS: Final = "sensors"
ATTR_BOILER_PARAMETERS: Final = "parameters"
ATTR_PENDING_ALERTS: Final = "pending_alerts"
ATTR_ALERTS: Final = "alerts"
ATTR_FAN_POWER: Final = "fan_power"
ATTR_FRAME_VERSIONS: Final = "frames"
ATTR_FUEL_CONSUMPTION: Final = "fuel_consumption"
ATTR_FUEL_LEVEL: Final = "fuel_level"
ATTR_FUEL_BURNED: Final = "fuel_burned"
ATTR_LOAD: Final = "load"
ATTR_MIXERS: Final = "mixers"
ATTR_MIXER_SENSORS: Final = "mixer_sensors"
ATTR_MIXER_PARAMETERS: Final = "mixer_parameters"
ATTR_MODE: Final = "mode"
ATTR_MODULES: Final = "modules"
ATTR_POWER: Final = "power"
ATTR_THERMOSTAT: Final = "thermostat"
ATTR_THERMOSTATS: Final = "thermostats"
ATTR_TRANSMISSION: Final = "transmission"
ATTR_PRODUCT: Final = "product"
ATTR_NETWORK: Final = "network"
ATTR_VERSION: Final = "version"
ATTR_PASSWORD: Final = "password"
ATTR_SCHEMA: Final = "schema"
ATTR_REGDATA: Final = "regdata"
ATTR_LAMBDA_SENSOR: Final = "lambda"
ATTR_SCHEDULES: Final = "schedules"
ATTR_TYPE: Final = "type"
ATTR_SWITCH: Final = "switch"
ATTR_PARAMETER: Final = "parameter"
ATTR_SCHEDULE: Final = "schedule"
